/*
Copyright (C) 2006-2008 Lin Himmelmann

This file is part of TreeTime.
See the NOTICE file distributed with this work for additional
information regarding copyright ownership and licensing.

TreeTime is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

TreeTime is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with TreeTime.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "GammaConstraint.hpp"
#include <gsl/gsl_rng.h>
#include <gsl/gsl_randist.h>

namespace TreeTime_Constraint
{
	std::ostream& operator<<(std::ostream& p_out, const GammaConstraint& p_Constr)
	{
		p_out << "list1={";
		for(set<int>::const_iterator I = p_Constr.m_sSpeciesId1.begin();
				I !=  p_Constr.m_sSpeciesId1.end(); ++I)
		{
			p_out << *I << " ";
		}
		p_out << "} list2={";
		for(set<int>::const_iterator I = p_Constr.m_sSpeciesId2.begin();
				I !=  p_Constr.m_sSpeciesId2.end(); ++I)
		{
			p_out << *I << " ";
		}
		p_out	<< "}"
				<< " minimum="	<< p_Constr.getMinimum()
				<< " mean="		<< p_Constr.getMean()
				<< " alpha="	<< p_Constr.getAlpha()
				<< " threshold="<< p_Constr.getThreshold();
		return p_out;
	}

	GammaConstraint::GammaConstraint(const set<int>& p_Ids1, const set<int>& p_Ids2,
					double p_minimum, double p_mean, double p_alpha, double p_threshold)
	: TopologicalConstraint	(p_Ids1, p_Ids2)
	, m_minimum			(p_minimum)
	, m_mean			(p_mean)
	, m_alpha			(p_alpha)
	, m_threshold		(p_threshold/(p_mean-p_minimum))
	{
	}

	GammaConstraint::~GammaConstraint()
	{
	}

	double GammaConstraint::getLogDensity(double p_timepoint) const
	{
		double logDensity = -10000.0;
		if(p_timepoint < m_minimum)
		{
			//return log(m_threshold);
			return logDensity;
		}
		try
		{
			 logDensity =log((gsl_ran_gamma_pdf(p_timepoint-m_minimum, m_alpha, (m_mean-m_minimum)/m_alpha))+m_threshold);
		}
		catch(...)
		{
			std::cout<< "Exception occured in function: GammaConstraint::getLogDensity(...)" << " ";
			std::cout<< p_timepoint << " " << m_alpha << std::endl;
		}
		return logDensity;
	}

}//TreeTime_Constraint
