/*
Copyright (C) 2006-2008 Lin Himmelmann

This file is part of TreeTime.
See the NOTICE file distributed with this work for additional
information regarding copyright ownership and licensing.

TreeTime is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

TreeTime is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with TreeTime.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef GAMMACONSTRAINT_HPP_
#define GAMMACONSTRAINT_HPP_

#include "Settings.hpp"
#include "TopologicalConstraint.hpp"

namespace TreeTime_Constraint
{

	class GammaConstraint;
	ostream& operator<<(ostream& , const GammaConstraint&);

	class GammaConstraint : public TopologicalConstraint
	{
		/**
		 * Declaration of friend ouput operator
		 */
		friend ostream& operator<<(ostream& , const GammaConstraint&);

		private:

			const double		m_minimum;
			const double		m_mean;
			const double		m_alpha;
			const double		m_threshold;

		public:

			GammaConstraint(const set<int>& p_Ids1, const set<int>& p_Ids2,
					double p_minimum, double p_mean, double p_alpha = 1.0,
					double p_threshold = 1.0E-100);
			virtual ~GammaConstraint();

			virtual inline bool isTimeConstraint() const;
			virtual double getLogDensity(double p_timepoint) const;
			inline const double getMinimum() const;
			inline const double getMean() const;
			inline const double getAlpha() const;
			inline const double getThreshold() const;

		private:

			GammaConstraint(const GammaConstraint&);
			GammaConstraint& operator = (const GammaConstraint&);

	};//GammaConstraint

}//TreeTime_Constraint

#include "GammaConstraint.cxx"

#endif /*GAMMACONSTRAINT_HPP_*/
