/*
Copyright (C) 2006-2008 Lin Himmelmann

This file is part of TreeTime.
See the NOTICE file distributed with this work for additional
information regarding copyright ownership and licensing.

TreeTime is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

TreeTime is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with TreeTime.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "GaussianConstraint.hpp"
#include <gsl/gsl_rng.h>
#include <gsl/gsl_randist.h>

namespace TreeTime_Constraint
{
	std::ostream& operator<<(std::ostream& p_out, const GaussianConstraint& p_Constr)
	{
		p_out << "list1={";
		for(set<int>::const_iterator I = p_Constr.m_sSpeciesId1.begin();
				I !=  p_Constr.m_sSpeciesId1.end(); ++I)
		{
			p_out << *I << " ";
		}
		p_out << "} list2={";
		for(set<int>::const_iterator I = p_Constr.m_sSpeciesId2.begin();
				I !=  p_Constr.m_sSpeciesId2.end(); ++I)
		{
			p_out << *I << " ";
		}
		p_out	<< "} mean="<< p_Constr.getMean()
				<< " sd="	<< p_Constr.getDeviation();
		return p_out;
	}

	GaussianConstraint::GaussianConstraint(const set<int>& p_Ids1, const set<int>& p_Ids2,
			double p_mean, double p_sd)
	: TopologicalConstraint	(p_Ids1, p_Ids2)
	, m_mean				(p_mean)
	, m_deviation			(p_sd)
	{
	}

	GaussianConstraint::~GaussianConstraint()
	{
	}

	double GaussianConstraint::getLogDensity(double p_timepoint) const
	{
		double logDensity = -10000.0;
		try
		{
			 logDensity = log(gsl_ran_gaussian_pdf(m_mean-p_timepoint, m_deviation));
		}
		catch(...)
		{
			std::cout<< "Exception occured in function: GaussianConstraint::getLogDensity(...)" <<  " ";
			std::cout<< p_timepoint << std::endl;
		}
		return logDensity;
	}

}//TreeTime_Constraint
