/*
Copyright (C) 2006-2008 Lin Himmelmann

This file is part of TreeTime.
See the NOTICE file distributed with this work for additional
information regarding copyright ownership and licensing.

TreeTime is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

TreeTime is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with TreeTime.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef GAUSSIANCONSTRAINT_HPP_
#define GAUSSIANCONSTRAINT_HPP_

#include "Settings.hpp"
#include "TopologicalConstraint.hpp"

namespace TreeTime_Constraint
{

	class GaussianConstraint;
	ostream& operator<<(ostream& , const GaussianConstraint&);

	class GaussianConstraint : public TopologicalConstraint
	{
		/**
		 * Declaration of friend ouput operator
		 */
		friend ostream& operator<<(ostream& , const GaussianConstraint&);

		private:

			const double		m_mean;
			const double		m_deviation;

		public:

			GaussianConstraint(const set<int>& p_Ids1, const set<int>& p_Ids2,
					double p_mean, double p_deviation);
			virtual ~GaussianConstraint();

			virtual inline bool isTimeConstraint() const;
			virtual double getLogDensity(double p_timepoint) const;
			inline const double getMean() const;
			inline const double getDeviation() const;

		private:

			GaussianConstraint(const GaussianConstraint&);
			GaussianConstraint& operator = (const GaussianConstraint&);

	};//GaussianConstraint

}//TreeTime_Constraint

#include "GaussianConstraint.cxx"

#endif /*GAUSSIANCONSTRAINT_HPP_*/
