/*
Copyright (C) 2006-2008 Lin Himmelmann

This file is part of TreeTime.
See the NOTICE file distributed with this work for additional
information regarding copyright ownership and licensing.

TreeTime is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

TreeTime is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with TreeTime.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef TreeTime_PROPOSALCHANGE_HPP_
#define TreeTime_PROPOSALCHANGE_HPP_

#include "Settings.hpp"
#include "../TreeTime_Mcmcmc/Change.hpp"

namespace TreeTime_Core
{

	class State;

	class ProposeChange
	{

		public:

			ProposeChange(double p_estimatedPemTime,
					bool p_hasTimeConstraints,
					double p_estimatedRootTime,
					double p_nSpecies,
					bool p_hasOutgroup,
					double p_weightTopology,
					double p_weightNodes,
					double p_weightEvolModels,
					double p_weightRateChangeModel,
					double p_weightInitialRates,
					double p_changeNodeTime,
					double p_changeTimeSynchronisation,
					double p_changeRootTime,
					State* p_pState);
			virtual ~ProposeChange();
			virtual TreeTime_Mcmcmc::Change* proposeNewChange(State* p_pState)
					const;

		private:

			ProposeChange(const ProposeChange&);
			ProposeChange& operator=(const ProposeChange&);
			
			TreeTime_Mcmcmc::Change* getMolecularEvolutionModelChange(
					State* p_pState, double p_uniform) const;
			TreeTime_Mcmcmc::Change* getRateChangeModelChange(
					State* p_pState, double p_uniform) const;

		private:

			const double			m_changeRootTimepointFactor;
			const double			m_changeInitialRatesFactor;
			const bool				m_hasOutgroup;
			const double			m_changeNodeTime;
			const double			m_changeTimeSynchronisation;
			const double			m_changeRootTime;
			const int				m_nLoci;
			vector<double>			m_vRcmWeigths;
			vector<vector<double> >	m_vvMemWeigths;
			vector<double>			m_vGroupWeights;

	};//ProposeChange

}//TreeTime_Core

#endif /*TreeTime_PROPOSALCHANGE_HPP_*/
