/*
Copyright (C) 2006-2008 Lin Himmelmann

This file is part of TreeTime.
See the NOTICE file distributed with this work for additional
information regarding copyright ownership and licensing.

TreeTime is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

TreeTime is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with TreeTime.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef RANDOMNUMBERGENERATOR_HPP_
#define RANDOMNUMBERGENERATOR_HPP_

#include "Settings.hpp"
#include "../TreeTime_MolecularEvolution/Random.hpp"
#include "../TreeTime_RateChangeModel/Random.hpp"
#include "../TreeTime_Mcmcmc/Random.hpp"
#include <gsl/gsl_rng.h>
#include <gsl/gsl_randist.h>

namespace TreeTime_Core
{

	class RandomNumberGenerator : public TreeTime_MolecularEvolution::Random,
			public TreeTime_Mcmcmc::Random,
			public TreeTime_RateChangeModel::Random
	{
			
		private:

			gsl_rng* m_pRNGTausworthe;

		public:

			static	RandomNumberGenerator& getRNG();
			void	setSeed(int p_seed);
			virtual double	getUniform() const;
			double	getUniform(double p_min, double p_max);
			virtual double	getUniform(double p_min, double p_max) const;
			virtual int		getInteger(int p_span) const;
			int		getInteger(int p_span, int p_minValue);
			double	getGaussian(double p_mean=0, double p_standardDeviation=1);
			double	getGaussianDensity(double p_value,
					double p_standardDeviation=1);
			double	getGamma(double p_alpha, double p_beta);
			double	getGammaDensity(double p_value, double p_alpha,
					double p_beta);
			unsigned int getPoisson(double p_mean);
			double	getPoissonDensity(unsigned int p_value, double p_mean);
			virtual void	getDirichlet(const double p_param[],
					double p_target[]) const;
			virtual double	getDirichletDensity(const double p_param[],
					const double p_target[]) const;
			void	getDirichlet(const double p_param[],
					double p_target[], int p_number) const;
			double	getLogDirichletDensity(const double p_param[],
					const double p_target[], int p_number) const;
			double	getExponential(double p_meanValue) const;
			double	getExponentialDensity(double p_value, double p_meanValue)
					const;
			double	getLognormal(double p_mu, double p_sigma) const;
			double	getLognormalDensity(double p_value, double p_mu,
					double p_sigma) const;

		private:

			RandomNumberGenerator();
			RandomNumberGenerator(const RandomNumberGenerator& rhs);
			RandomNumberGenerator& operator=(const RandomNumberGenerator& rhs);
			virtual ~RandomNumberGenerator();

	};//RandomNumberGenerator

}//TreeTime_Core
#endif /*RANDOMNUMBERGENERATOR_HPP_*/
