/*
Copyright (C) 2006-2008 Lin Himmelmann

This file is part of TreeTime.
See the NOTICE file distributed with this work for additional
information regarding copyright ownership and licensing.

TreeTime is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

TreeTime is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with TreeTime.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef STATE_HPP_
#define STATE_HPP_

#include "Settings.hpp"
#include "NodeData.hpp"
#include "../TreeTime_Constraint/TopologicalConstraint.hpp"
#include "../TreeTime_MolecularEvolution/DnaEvolutionModel.hpp"
#include "../TreeTime_MolecularTree/Tree.hpp"

namespace TreeTime_RateChangeModel
{

	template<class x>
	class RateChangeModel;

}//TreeTime_RateChangeModel

namespace TreeTime_Core
{

	struct State
	{

		typedef TreeTime_MolecularEvolution::DnaEvolutionModel		DnaModel;
		typedef TreeTime_Constraint::TopologicalConstraint			Constraint;
		typedef TreeTime_MolecularTree::Tree<NodeData>				Phylogeny;	
		typedef TreeTime_RateChangeModel::RateChangeModel<State>	RCModel;

		double						m_totalLogLikelihood;
		double						m_treeLogLikelihood;
		double						m_constraintLogLikelihood;
		double						m_globalRateLoglikelihood;
		vector<double>				m_vLocalRateLoglikelihood;
		Phylogeny*					m_pTree;
		vector<double>				m_vRateScaling;
		vector<DnaModel*> 			m_vpDnaEvolutionModel;
		RCModel*					m_pGlobalRateChangeModel;
		vector<RCModel*>			m_vpLocalRateChangeModel;
		static vector<Constraint*>	s_vpConstraint;

	};//State

}//TreeTime_Core

#endif /*STATE_HPP_*/
