/*
Copyright (C) 2006-2008 Lin Himmelmann

This file is part of TreeTime.
See the NOTICE file distributed with this work for additional
information regarding copyright ownership and licensing.

TreeTime is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

TreeTime is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with TreeTime.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef CHANGEINITIALRATES_HPP_
#define CHANGEINITIALRATES_HPP_

#include "../../TreeTime_Mcmcmc/Change.hpp"
#include "../State.hpp"

namespace TreeTime_Core
{

	class ChangeInitialRates : public TreeTime_Mcmcmc::Change
	{

		public:

			ChangeInitialRates(State* p_pState, double p_proposalRatio,
					double p_scaling, double p_calibration);
			virtual ~ChangeInitialRates();
			virtual void	changeState();
			virtual double	getStationaryDistributionRatio() const;
			virtual double	getProposalRatio() const;
			virtual void	accept();
			virtual void	reject();
			virtual int		getId () const;

		private:

			void allocateSubtreeMemory(TreeTime_MolecularTree::iterator<NodeData> p_iterator);

		private:

			State*					m_pState;
			double					m_proposalRatio;
			const double			m_scaling;
			TreeTime_MolecularTree::Tree<NodeData>*	m_pProposalTree;
			vector<double>			m_vRateScaling;
			double					m_proposalTreeLogLikelihood;
			double					m_calibration;
			double					m_logPriorRatio;
			list<realf****>		m_lpCurrentProbablities;
			list<realf****>		m_lpProposalProbablities;

	};//ChangeInitialRates

}//TreeTime_Core

#endif /*CHANGEINITIALRATES_HPP_*/
