/*
Copyright (C) 2006-2008 Lin Himmelmann

This file is part of TreeTime.
See the NOTICE file distributed with this work for additional
information regarding copyright ownership and licensing.

TreeTime is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

TreeTime is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with TreeTime.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "ChangeMemInvSites.hpp"
#include "../RandomNumberGenerator.hpp"
#include "../LikelihoodCalculations.hpp"

namespace TreeTime_Core
{

	ChangeMemInvSites::ChangeMemInvSites(State* p_pState,double p_proposalRatio,
			int p_modelNr)
	: m_modelNr						(p_modelNr < 0 ? 0 : p_modelNr)
	, m_pState						(p_pState)
	, m_proposalRatio				(p_proposalRatio)
	, m_propOfInvSites				(1)
	, m_proposalTreeLogLikelihood	(0)
	{
	}

	ChangeMemInvSites::~ChangeMemInvSites()
	{
	}

	void ChangeMemInvSites::changeState()
	{
		m_propOfInvSites = m_pState->m_vpDnaEvolutionModel[m_modelNr]->
				getProportionOfInvariantSites();
		m_proposalRatio *= m_pState->m_vpDnaEvolutionModel[m_modelNr]->
				changeInvariantSites();
		m_proposalTreeLogLikelihood =
				LikelihoodCalculations::getLogLikelihoodTree(
				m_pState->m_pTree, m_pState->m_vpDnaEvolutionModel);
	}

	double ChangeMemInvSites::getStationaryDistributionRatio() const
	{
		return exp(m_proposalTreeLogLikelihood - m_pState->m_treeLogLikelihood);
	}

	double ChangeMemInvSites::getProposalRatio() const
	{
		return m_proposalRatio;
	}

	void ChangeMemInvSites::accept()
	{
		m_pState->m_totalLogLikelihood += (m_proposalTreeLogLikelihood -
				m_pState->m_treeLogLikelihood);
		m_pState->m_treeLogLikelihood = m_proposalTreeLogLikelihood;
	}

	void ChangeMemInvSites::reject()
	{
		m_pState->m_vpDnaEvolutionModel[m_modelNr]->
				setProportionOfInvariantSites(m_propOfInvSites);
	}

	int ChangeMemInvSites::getId () const
	{
		return 7;
	}

}//TreeTime_Core
