/*
Copyright (C) 2006-2008 Lin Himmelmann

This file is part of TreeTime.
See the NOTICE file distributed with this work for additional
information regarding copyright ownership and licensing.

TreeTime is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

TreeTime is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with TreeTime.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef CHANGETOPOLOGY_HPP_
#define CHANGETOPOLOGY_HPP_

#include "../../TreeTime_Mcmcmc/Change.hpp"
#include "../State.hpp"

namespace TreeTime_Core
{

	class ChangeTopology : public TreeTime_Mcmcmc::Change
	{

		public:

			ChangeTopology(State* p_pState, double p_proposalRatio,
					bool p_hasOutgroup);
			virtual ~ChangeTopology();
			virtual void	changeState();
			virtual double	getStationaryDistributionRatio() const;
			virtual double	getProposalRatio() const;
			virtual void	accept();
			virtual void	reject();
			virtual int		getId () const;

		private:

			void allocateSubtreeMemory(TreeTime_MolecularTree::
					iterator<NodeData> p_iterator);
			void adjustSubtreeInitialRates(TreeTime_MolecularTree::iterator<
					NodeData> p_iterator, int p_loci, double p_multiplier);

		private:

			State*					m_pState;
			double					m_proposalRatio;
			bool					m_hasOutgroup;
			TreeTime_MolecularTree::Tree<NodeData>*	m_pProposalTree;
			double					m_proposalTreeLogLikelihood;
			double					m_proposalGlobalRateLogLikelihood;
			vector<double>			m_vProposalLocalRateLoglikelihood;
			double					m_proposalConstraintLogLikelihood;
			double					m_proposalTotalLogLikelihood;
			list<realf****>			m_lpCurrentProbablities;
			list<realf****>			m_lpProposalProbablities;

	};//ChangeTopology

}//TreeTime_Core

#endif /*CHANGETOPOLOGY_HPP_*/
