/*
Copyright (C) 2006-2008 Lin Himmelmann

This file is part of TreeTime.
See the NOTICE file distributed with this work for additional
information regarding copyright ownership and licensing.

TreeTime is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

TreeTime is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with TreeTime.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "StateManager.hpp"
#include "StateSampler.hpp"
#include "ProposeChange.hpp"
#include "../TreeTime_RateChangeModel/RateChangeModel.hpp"

namespace TreeTime_Core
{

	ProposeChange*	StateManager::s_pProposeChange;
	StateSampler*	StateManager::s_pStateSampler;

	StateManager::StateManager(State* p_pState)
	: m_pState	(new State(*p_pState))
	{
		if(p_pState->m_pTree != 0)
		{
			m_pState->m_pTree = p_pState->m_pTree->copy();
		}
		for(unsigned int i=0; i < m_pState->m_vpDnaEvolutionModel.size() ;++i)
		{
			m_pState->m_vpDnaEvolutionModel[i] = 
					p_pState->m_vpDnaEvolutionModel[i]->clone();
		}
		m_pState->m_pGlobalRateChangeModel = 
				p_pState->m_pGlobalRateChangeModel->clone();
		for(unsigned int i=0; i < m_pState->m_vpDnaEvolutionModel.size() ;++i)
		{
			m_pState->m_vpLocalRateChangeModel[i] = 
					p_pState->m_vpLocalRateChangeModel[i]->clone();
		}
	}

	StateManager::StateManager(const StateManager& p_StateManager)
	: m_pState (new State(*p_StateManager.m_pState))
	{
		if(p_StateManager.m_pState->m_pTree != 0)
		{
			m_pState->m_pTree = p_StateManager.m_pState->m_pTree->copy();
		}
		for(unsigned int i=0; i < m_pState->m_vpDnaEvolutionModel.size() ;++i)
		{
			m_pState->m_vpDnaEvolutionModel[i] = 
					p_StateManager.m_pState->m_vpDnaEvolutionModel[i]->clone();
		}
		m_pState->m_pGlobalRateChangeModel = 
				p_StateManager.m_pState->m_pGlobalRateChangeModel->clone();
		for(unsigned int i=0; i < m_pState->m_vpDnaEvolutionModel.size() ;++i)
		{
			m_pState->m_vpLocalRateChangeModel[i] = p_StateManager.m_pState->
					m_vpLocalRateChangeModel[i]->clone();
		}
	}

	StateManager::~StateManager()
	{
		delete m_pState->m_pTree;
		for(unsigned int i=0; i < m_pState->m_vpDnaEvolutionModel.size() ;++i)
			delete m_pState->m_vpDnaEvolutionModel[i];
		delete m_pState->m_pGlobalRateChangeModel;
		for(unsigned int i=0; i < m_pState->m_vpDnaEvolutionModel.size() ;++i)
			delete m_pState->m_vpLocalRateChangeModel[i];
		delete m_pState;
	}

	double StateManager::getLogStationaryDistribution() const
	{
		return m_pState->m_totalLogLikelihood;
	}

	void StateManager::sample() const
	{
		s_pStateSampler->sample(m_pState);
	}

	void StateManager::proposeNewChange()
	{
		m_pChange = s_pProposeChange->proposeNewChange(m_pState);
	}

	TreeTime_Mcmcmc::Change*	StateManager::getChange() const
	{
		return m_pChange;
	}

	void StateManager::setStateSampler(StateSampler* p_pStateSampler)
	{
		s_pStateSampler = p_pStateSampler;
	}

	void StateManager::setProposeChange(ProposeChange* p_pProposeChange)
	{
		s_pProposeChange = p_pProposeChange; 
	}

	State* StateManager::getState()
	{
		return m_pState;
	}

}//TreeTime_Core
