/*
Copyright (C) 2006-2008 Lin Himmelmann

This file is part of TreeTime.
See the NOTICE file distributed with this work for additional
information regarding copyright ownership and licensing.

TreeTime is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

TreeTime is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with TreeTime.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef BTSTATEMANAGER_HPP_
#define BTSTATEMANAGER_HPP_

#include "Settings.hpp"
#include "State.hpp"
#include "../TreeTime_Mcmcmc/Change.hpp"
#include "../TreeTime_Mcmcmc/StateManager.hpp"

namespace TreeTime_Core
{
	class StateSampler;
	class ProposeChange;

	class StateManager : public TreeTime_Mcmcmc::StateManager
	{

		public:

			StateManager(State* p_pState);
			StateManager(const StateManager&);
			StateManager& operator=(const StateManager&);
			virtual ~StateManager();

			virtual double	getLogStationaryDistribution() const;
			virtual void	sample() const;
			virtual void	proposeNewChange();
			virtual TreeTime_Mcmcmc::Change*	getChange() const;

			static void		setStateSampler(StateSampler* p_pStateSampler);
			static void		setProposeChange(ProposeChange* p_pProposeChange);

			State*			getState();

		private:

			static StateSampler*	s_pStateSampler;
			static ProposeChange*	s_pProposeChange;
			State*					m_pState;

	};//StateManager

}//TreeTime_Core

#endif /*BTSTATEMANAGER_HPP_*/
