/*
Copyright (C) 2006-2008 Lin Himmelmann

This file is part of TreeTime.
See the NOTICE file distributed with this work for additional
information regarding copyright ownership and licensing.

TreeTime is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

TreeTime is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with TreeTime.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef TreeTime_STATESAMPLER_HPP_
#define TreeTime_STATESAMPLER_HPP_

#include "Settings.hpp"
#include "../TreeTime_MolecularTree/iterator.hpp"
#include "NodeData.hpp"
#include "../TreeTime_Mcmcmc/Sampler.hpp"

namespace TreeTime_Core
{

	class State;

	class StateSampler : public TreeTime_Mcmcmc::Sampler
	{

		public:

			StateSampler(map<int,string>& p_mIdToName, string p_outputDir,
					const int p_nChains);
			virtual ~StateSampler();

			void sample(const State* const p_pState);

			virtual void finishOutput();
			virtual void monitorChange(const int p_chain, const int p_changeId,
					const bool p_isAccepted);
			virtual void monitorSwap(const int p_chain, const int p_chain2,
					const bool p_isAccepted);

		private:

			string	getTreeUM(TreeTime_MolecularTree::iterator<NodeData>);
			string	getTreeUR(TreeTime_MolecularTree::iterator<NodeData>,
					int p_loci);
			double	getScaledLength(TreeTime_MolecularTree::iterator<NodeData>,
					int p_loci);
			void	getChangePointValues(TreeTime_MolecularTree::
					iterator<NodeData> p_I, list<double>& p_lChangePoints);

		private:

			map<int,string> 				m_mIdToName;
			string							m_outputDir;
			const int						m_nChains;
			vector<vector<pair<int,int> > > m_countSwappings;
			vector<vector<pair<int,int> > > m_countChanges;
			const std::ios::openmode		m_mode;
			ofstream						m_outTreesUM;
			ofstream						m_outTreesUR;
			ofstream						m_outState;
			ofstream						m_outRcms;
			ofstream						m_outMems;
			//ofstream						m_outCp;
			ofstream						m_outChanges;
			ofstream						m_outSwaps;
			int								m_nrOfSample;
			

	};//StateSampler

}//TreeTime_Core

#endif /*TreeTime_STATESAMPLER_HPP_*/
