/*
Copyright (C) 2006-2008 Lin Himmelmann

This file is part of TreeTime.
See the NOTICE file distributed with this work for additional
information regarding copyright ownership and licensing.

TreeTime is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

TreeTime is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with TreeTime.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef GTRMODEL_HPP_
#define GTRMODEL_HPP_

#include "../DnaEvolutionModel.hpp"

namespace TreeTime_MolecularEvolution
{

	class GtrModel: public DnaEvolutionModel
	{

		public:

			/**
			 * Constructor
			 */
			GtrModel(double p_piA, double p_piC, double p_piG, double p_piT,
					double p_x1, double p_x2, double p_x3, double p_x4,
					double p_x5, double p_x6, bool p_hasInvSites,
					double p_propOfInvSites, int p_nGammaRates,
					double p_gammaShapeParameter);

			/**
			 * Copy Constructor
			 */
			GtrModel(const GtrModel& p_Rhs);

			/**
			 * Destructor
			 */
			virtual ~GtrModel();

			/**
			 * Clones the Jukes Cantor Model
			 * @return Pointer to new Allocated Jukes Cantor Model
			 */
			virtual GtrModel* clone() const;

			/**
			 * Returns the Transition Matrix for a given Time Distance
			 */
			virtual double** const getTransitionMatrix(double p_time);

			/**
			 *  Changes Stationary Distribution, returns the Proposal Density
			 */
			virtual double changeStationaryDistribution();

			/**
			 *  Changes Transitionmatrix Parameter, returns the Proposal Density
			 */
			virtual double changeTransitionParameters();

		private:

			/**
			 * Prints the actual Model on the given Output Stream
			 */
			virtual void operator<<(std::ostream&) const;

			/**
			 * Not implemented
			 */
			GtrModel& operator=(const GtrModel& rhs);
		
			void updateRateMatrix();
			void updateParametersTo1Pem();


		private:

			double* const	m_parameters;
			double** const	m_rateMatrix;

	};//GtrModel

}//TreeTime_MolecularEvolution

#endif /*GTRMODEL_HPP_*/
