/*
Copyright (C) 2006-2008 Lin Himmelmann

This file is part of TreeTime.
See the NOTICE file distributed with this work for additional
information regarding copyright ownership and licensing.

TreeTime is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

TreeTime is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with TreeTime.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "JukesCantorModel.hpp"
#include "../Helper/GammaRates.hpp"

namespace TreeTime_MolecularEvolution
{

	JukesCantorModel::JukesCantorModel(bool p_hasInvSites,
			double p_propOfInvSites, int p_nGammaRates,
			double p_gammaShapeParameter)
	: DnaEvolutionModel(p_hasInvSites,p_propOfInvSites,p_nGammaRates,
			p_gammaShapeParameter,0,0)
	{
		m_stationaryDistribution[0] = m_stationaryDistribution[1] = 
		m_stationaryDistribution[2] = m_stationaryDistribution[3] = .25;
	}

	JukesCantorModel::JukesCantorModel(const JukesCantorModel& p_Rhs)
	: DnaEvolutionModel(p_Rhs)
	{
	}

	JukesCantorModel::~JukesCantorModel()
 	{
 	}

	JukesCantorModel* JukesCantorModel::clone() const
	{
		JukesCantorModel* newJCM = new JukesCantorModel(*this);
		return newJCM;
	}

 	double** const JukesCantorModel::getTransitionMatrix(double p_time)
	{
		double alpha = (1.0-exp(-p_time*4.0/300.0))/4.0;
		double diag  = 1.0 - 3.0*alpha;
		for(int i = 0; i<4; ++i)
		{
			for(int j = 0; j<4; ++j)
			{
				if(i==j)
					m_transitionMatrix[i][j] = diag;
				else
					m_transitionMatrix[i][j] = alpha;	
			}
		}
		return m_transitionMatrix;
	}

	double JukesCantorModel::changeStationaryDistribution()
	{
		return 1;
	}

	double JukesCantorModel::changeTransitionParameters()
	{
		return 1;
	}

	void JukesCantorModel::operator<<(std::ostream& p_out) const
	{
		p_out << "JC\t";
		p_out << m_nGammaRates					<< "\t";
		p_out << m_gammaShapeParameter			<< "\t";
		p_out << m_hasInvariantSites			<< "\t";
		p_out << m_proportionOfInvariantSites	<< "\t";
		for(unsigned int i = 0;i<3;++i)
			p_out << m_stationaryDistribution[i]<< "\t";
		p_out << m_stationaryDistribution[3];	
	}

}//TreeTime_MolecularEvolution
