/*
Copyright (C) 2006-2008 Lin Himmelmann

This file is part of TreeTime.
See the NOTICE file distributed with this work for additional
information regarding copyright ownership and licensing.

TreeTime is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

TreeTime is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with TreeTime.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef JUKESCANTORMODEL_HPP_
#define JUKESCANTORMODEL_HPP_

#include "../DnaEvolutionModel.hpp"

namespace TreeTime_MolecularEvolution
{
	/**
	 * Jukes Cantor Model 1969
	 */
	class JukesCantorModel: public DnaEvolutionModel
	{

		public:
			
			/**
			 * Constructor
			 */
			JukesCantorModel(bool p_hasInvSites, double p_propOfInvSites,
					int p_nGammaRates, double p_gammaShapeParameter);

			/**
			 * Copy Constructor
			 */
			JukesCantorModel(const JukesCantorModel& rhs);

			/**
			 * Destructor
			 */
			virtual ~JukesCantorModel();
			
			/**
			 * Clones the Jukes Cantor Model
			 * @return Pointer to new Allocated Jukes Cantor Model
			 */
			virtual JukesCantorModel*	clone() const;

			/**
			 * Returns the Transition Matrix for a given Time Distance
			 */
			virtual double** const		getTransitionMatrix(double p_time);
			
			/**
			 *  Does nothing, returns 1
			 */
			virtual double				changeStationaryDistribution();

			/**
			 *  Does nothing, returns 1
			 */
			virtual double				changeTransitionParameters();

		private:
			
			/**
			 * Not implemented
			 */
			JukesCantorModel& operator=(const JukesCantorModel& rhs);
			
			/**
			 * Prints the actual Model on the given Output Stream
			 */
			virtual void operator<<(std::ostream&) const;

	};//JukesCantorModel

}//TreeTime_MolecularEvolution

#endif /*JUKESCANTORMODEL_HPP_*/

