/*
Copyright (C) 2006-2008 Lin Himmelmann

This file is part of TreeTime.
See the NOTICE file distributed with this work for additional
information regarding copyright ownership and licensing.

TreeTime is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

TreeTime is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with TreeTime.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "GammaRates.hpp"
#include <gsl/gsl_cdf.h>

namespace TreeTime_MolecularEvolution
{

	vector<double> GammaRates::getGammaRates(int p_nRates, double
			p_gammaShapeParameter)
	{
		// assert(p_gammaShapeParameter >= 0.05);
		// assert(p_gammaShapeParameter <= 50.0);
		vector<double> vGammaRates;
		if(p_nRates<1)
		{
			return vGammaRates;
		}
		else if(p_nRates<2)
		{
			vGammaRates.push_back(1.0);
			return vGammaRates;
		}
		else
		{
			double normalisation = 0;
			for (int i=0; i<p_nRates; ++i)
			{
				double temp = 1.0;
				try
				{
					temp = gsl_cdf_gamma_Pinv((static_cast<double>(i) + 0.5)/static_cast<double>(p_nRates),p_gammaShapeParameter, 1.0/p_gammaShapeParameter);
				}
				catch(...)
				{
					std::cout<< "Exception occured in function: GammaRates::getGammaRates(...)" << " ";
					std::cout<< p_nRates << " " << p_gammaShapeParameter << " " << temp << std::endl;
				}
				vGammaRates.push_back(temp);
				normalisation += temp;
			}
			for(std::vector<double>::iterator I = vGammaRates.begin(); I != vGammaRates.end(); ++I)
			{
				*I = (*I/normalisation)*static_cast<double>(p_nRates);
			}
			return vGammaRates;
		}
	}

}//TreeTime_MolecularEvolution
