/*
Copyright (C) 2006-2008 Lin Himmelmann

This file is part of TreeTime.
See the NOTICE file distributed with this work for additional
information regarding copyright ownership and licensing.

TreeTime is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

TreeTime is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with TreeTime.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef GAMMARATES_HPP_
#define GAMMARATES_HPP_

#include "../Settings.hpp"

namespace TreeTime_MolecularEvolution
{
	/**
	 * Calculates a discrete Approximation of varying Rates
	 */
	class GammaRates
	{

		public:

			/**
			 * Calculater an array of Length p_nRates, with discrete Rate
			 * Approximation parametrisiesd by p_gammaShapeParameter.
			 * The Gamma Shape Parameter must be in [0.05,50].
			 * Bigger values results in more equal values.
			 * @param int
			 * @param double
			 * @return vector<double>
			 */
			static vector<double> getGammaRates(int p_nRates, double p_gammaShapeParameter);

		private:

			GammaRates();
			GammaRates(const GammaRates&);
			GammaRates& operator=(const GammaRates&);
			~GammaRates();

	};//GammaRates
	
}//TreeTime_MolecularEvolution 

#endif /*GAMMARATES_HPP_*/
