/*
Copyright (C) 2006-2008 Lin Himmelmann

This file is part of TreeTime.
See the NOTICE file distributed with this work for additional
information regarding copyright ownership and licensing.

TreeTime is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

TreeTime is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with TreeTime.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef BINARYNODE_HPP_
#define BINARYNODE_HPP_

namespace TreeTime_MolecularTree
{

	///BinaryNode represents a binary Node in the BinaryTree.
	/**
	 * The Interface consists only of getters and setters for the Attributes.
	 * The Atributes are Pointers to Father, RightSon and LeftSon,
	 * and Pointers to NodeData and MolecularData.
	 */
	template<class NODEDATA>
	class BinaryNode
	{

		public:
			/**
			 * Constructor that creates a Node with standard constructed
			 * NodeData
			 */
			BinaryNode();
			/**
			 * The Copy Constructor do not copys the Pointers to the Father and Child
			 * Nodes, and uses the CopyConstructor for NodeData.
			 */
			BinaryNode(const BinaryNode&);
			/**
			 * This Constructor initialises all Pointers  to BinaryNodes with 0
			 * and uses the CopyConstructor for NodeData.
			 */
			BinaryNode(const NODEDATA&);
			/**
			 * The Assignment Operator do not copy the Pointers to the Father and
			 * Child Nodes, and uses the CopyConstructor for NodeData.
			 */
			BinaryNode& operator=(const BinaryNode&);
			/**
			 * The Destructor just calls the Destructor for the NodeData
			 */
			~BinaryNode();
			/**
			 * Getter Method for getting the Father Node;
			 * @return Father Node
			 */
			inline BinaryNode*	getFather()		const;
			/**
			 * Getter Method for getting the Right Son Node;
			 * @return Rigth Son Node
			 */
			inline BinaryNode*	getRightSon()	const;
			/**
			 * Getter Method for getting the Left Son Node;
			 * @return Left Son Node
			 */
			inline BinaryNode*	getLeftSon()	const;
			/**
			 * Setter Method for setting the Father Node;
			 * @param new Father Node
			 */
			inline void			setFather(BinaryNode*	p_pNewFather);
			/**
			 * Setter Method for setting the Right Son Node;
			 * @param new Right Son Node
			 */
			inline void			setRightSon(BinaryNode*	p_pNewRightSon);
			/**
			 * Setter Method for setting the Left Son Node;
			 * @param new Left Son Node
			 */
			inline void			setLeftSon(BinaryNode*	p_pNewLeftSon);
			/**
			 * Getter Method for getting the NodeData;
			 * @return Reference to the NodeData
			 */
			inline NODEDATA& 	getNodeData();

		private:
			BinaryNode*		m_pFather;  	/**< Pointer to the Father */
			BinaryNode* 	m_pRightSon;	/**< Pointer to the right Son */
			BinaryNode* 	m_pLeftSon;		/**< Pointer to the left Son */
			NODEDATA		m_NodeData;		/**< NodeData */

	};//BinaryNode

}//TreeTime_MolecularTree

#include "BinaryNode.cxx"

#endif /*BINARYNODE_HPP_*/
