/*
Copyright (C) 2006-2008 Lin Himmelmann

This file is part of TreeTime.
See the NOTICE file distributed with this work for additional
information regarding copyright ownership and licensing.

TreeTime is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

TreeTime is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with TreeTime.  If not, see <http://www.gnu.org/licenses/>.
*/
namespace TreeTime_MolecularTree
{
	template<class NODEDATA>
	BinaryTree<NODEDATA>::BinaryTree(const string& p_newickString,
			map<string,int>& p_mSpeciesToId)
	{
		BinaryNode<NODEDATA>* pRootNode  = new BinaryNode<NODEDATA>;
		m_iRoot = iterator<NODEDATA>(pRootNode);
		BinaryNode<NODEDATA>* aZ = pRootNode;//Initialise Pointer to the Root
		for(string::const_iterator I = p_newickString.begin();
				I != p_newickString.end(); ++I)
		{
			if ( *I == '(' )//Begin new subtree
			{
				BinaryNode<NODEDATA>* p = new BinaryNode<NODEDATA>;
				p->setFather(aZ);
				if(aZ->getLeftSon() == 0)
					aZ->setLeftSon(p);
				else
					aZ->setRightSon(p);
				//aZ->setLeftSon(p);
				aZ = p;
			}
			else if( *I == ')' )// Subtree finished, get back
			{
				aZ = aZ->getFather();
			}
			else if( *I == ',' )// insert brother
			{
				BinaryNode<NODEDATA>* p = new BinaryNode<NODEDATA>;
				aZ = aZ->getFather();
				p->setFather(aZ);
				if(aZ->getLeftSon() == 0)
					aZ->setLeftSon(p);
				else
					aZ->setRightSon(p);
				//aZ->setRightSon(p);
				aZ = p;
			}
			else if( *I == ':' )// treelength
			{
				std::string length = "";
				++I;
				while( *I!=',' && *I!=')' && *I!=':' && *I!='(' && *I!=';')
				{
					length += *I;
					++I;
				}
				--I;
				aZ->getNodeData().m_length = atof(length.c_str());
			}
			else if( *I == ';' )
			{
				break;
			}
			else// name
			{
				std::string name = "";
				do
				{
					name += *I;
					++I;
				}
				while( *I!=',' && *I!=')' && *I!=':' && *I!='(' && *I!=';');
				--I;
				aZ->getNodeData().m_id = p_mSpeciesToId[name];
			}
		}
	}

	template<class NODEDATA>
	BinaryTree<NODEDATA>::BinaryTree(const BinaryTree<NODEDATA>& p_BinaryTree)
	{
		iterator<NODEDATA> iMaster = p_BinaryTree.getRoot();
		if(iMaster)
		{
			BinaryNode<NODEDATA>* pRootNode  = new BinaryNode<NODEDATA>(*iMaster);
			m_iRoot = iterator<NODEDATA>(pRootNode);
			copy(iMaster.m_pNode,pRootNode);
		}
	}

	template<class NODEDATA>
	void BinaryTree<NODEDATA>::copy(BinaryNode<NODEDATA>* p_pMaster, BinaryNode<NODEDATA>* p_pReplica)
	{
		BinaryNode<NODEDATA>* pMasterChild = p_pMaster->getLeftSon();
		if(pMasterChild != 0)
		{
			BinaryNode<NODEDATA>* pReplicaChild = new BinaryNode<NODEDATA>(*pMasterChild);
			pReplicaChild->setFather(p_pReplica);
			p_pReplica->setLeftSon(pReplicaChild);
			copy(pMasterChild,pReplicaChild);
		}
		pMasterChild = p_pMaster->getRightSon();
		if(pMasterChild != 0)
		{
			BinaryNode<NODEDATA>* pReplicaChild = new BinaryNode<NODEDATA>(*pMasterChild);
			pReplicaChild->setFather(p_pReplica);
			p_pReplica->setRightSon(pReplicaChild);
			copy(pMasterChild,pReplicaChild);
		}
	}

	template<class NODEDATA>
	void BinaryTree<NODEDATA>::remove(BinaryNode<NODEDATA>* p_pNode)
	{
		BinaryNode<NODEDATA>* pChildNode = p_pNode->getLeftSon();
		if(pChildNode != 0)
		{
			remove(pChildNode);
		}
		pChildNode = p_pNode->getRightSon();
		if(pChildNode != 0)
		{
			remove(pChildNode);
		}
		delete p_pNode;
	}

	template<class NODEDATA>
	BinaryTree<NODEDATA>::~BinaryTree()
	{
		if(m_iRoot)
			remove(m_iRoot.m_pNode);
	}

	template<class NODEDATA>
	iterator<NODEDATA> BinaryTree<NODEDATA>::BinaryTree::getRoot() const
	{
		return m_iRoot;
	}

	template<class NODEDATA>
	void BinaryTree<NODEDATA>::setRoot(iterator<NODEDATA> p_rootIterator)
	{
		m_iRoot = p_rootIterator;
	}

	template<class NODEDATA>
	Tree<NODEDATA>* BinaryTree<NODEDATA>::copy()
	{
		return new BinaryTree<NODEDATA>(*this);
	}

}//TreeTime_MolecularTree
