/*
Copyright (C) 2006-2008 Lin Himmelmann

This file is part of TreeTime.
See the NOTICE file distributed with this work for additional
information regarding copyright ownership and licensing.

TreeTime is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

TreeTime is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with TreeTime.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef TREE_HPP_
#define TREE_HPP_

#include "Settings.hpp"
#include "iterator.hpp"
#include "NodeData.hpp"

namespace TreeTime_MolecularTree
{

//	class iterator;// Foreward declaration

	/**
	 * @brief Tree Interface.
	 * 
	 * Tree Interface.
	 * Access to the NodeData via the Iterator iterator.
	 */
	template<class NODEDATA>
	class Tree
	{
		public:
			/**
			 * Copys a Tree
			 * @return Pointer to the new allocated Tree
			 */
			virtual Tree* copy() = 0;
			/**
			 * Returns an Iterator to the firts Element (Root) of the Tree
			 * @return Iterator to the first Element
			 */
			virtual iterator<NODEDATA> getRoot() const = 0;
			/**
			 * Sets the Root Node of the Tree, given in the Argument.
			 * @param Iterator to the new Root Node
			 */
			virtual void setRoot(iterator<NODEDATA> p_rootIterator)=0;
			
			virtual ~Tree();
	};//Tree

}//TreeTime_MolecularTree

#include "Tree.cxx"

#endif /*TREE_HPP_*/
