/*
Copyright (C) 2006-2008 Lin Himmelmann

This file is part of TreeTime.
See the NOTICE file distributed with this work for additional
information regarding copyright ownership and licensing.

TreeTime is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

TreeTime is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with TreeTime.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "NexusParser.hpp"
#include "Data.hpp"

namespace TreeTime_NexusProcessing
{

	NexusParser::NexusParser(Data* CurrentData_):
	CurrentData(CurrentData_)
	{
	}


	void NexusParser::parseNexusFile(string& nexusfile)
	{
		// Nexus File einlesen und Kommentare entfernen:
		std::stringstream StringStream;
		int brackets = 0;
		std::ifstream ifs(nexusfile.c_str());
		while(!ifs)
		{
			ifs.clear();
			string in;
			cout << "Please specify correct input-filename." << endl;
			std::getline(std::cin, in);
			ifs.open(in.c_str());
		}
		string buf;
		cout << "Removing comments:" << endl;
		while(getline(ifs,buf))
		{
			string line="";
			while(buf.size()>0)
			{
				if(positionOfBracket(buf)==0)
				{
					if(buf.at(0)=='[')
						brackets ++;
					else
						brackets --;
					buf = buf.substr(1);
				}
				else if(positionOfBracket(buf)>0)
				{
					int pos = positionOfBracket(buf);
					if(brackets==0)
						line += buf.substr(0,pos);
					if(buf.at(pos)=='[')
						brackets ++;
					else
						brackets --;
					buf = buf.substr(pos+1);
				}
				else
				{
					if(brackets==0)
						line += buf;
					break;
				}
			}
			if(line != "")
			{
				StringStream << line << std::endl;
			}
		}
		cout << "Removing comments finished." << endl;
		std::stringstream StringStream2(StringStream.str());
		// Nexus Bloecke auslesen:
		while(StringStream >> buf)
		{
			if(buf=="begin" || buf=="Begin" || buf=="BEGIN")
			{
				StringStream >> buf;
				if(buf=="data;" || buf=="Data;" || buf=="DATA;")
				{
					processNexusDataBlock(StringStream);
				}
			}
		}
		while(StringStream2 >> buf)
		{
			if(buf=="begin" || buf=="Begin" || buf=="BEGIN")
			{
				StringStream2 >> buf;
				if(buf=="treetime;" || buf=="Treetime;" || buf=="TreeTime;" || buf=="TREETIME;")
				{
					processNexusTreeTimeBlock(StringStream2);
				}
			}
		}
		// Finishing Parsing:
		CurrentData->preprocessDnaData();
		CurrentData->checkInitialTree();
	}
	
	
	
	
	void NexusParser::processNexusDataBlock(std::stringstream& StringStream)
	{
		cout << "Parsing data-block:" << endl;
		bool interleave = false;
		string buf;
		StringStream >> buf;
		while(buf!="end;" && buf!="End;" && buf!="END;")
		{
			if(buf=="dimensions")
			{
				while(true)
				{
					StringStream >> buf;
					if(buf==";")break;
					string variable = buf.substr(0,buf.find_first_of("=;"));
					string wert = "";
					if(buf.find_first_of("=")<string::npos) //falls = zeichen enthalten...
					{
						wert = buf.substr(buf.find_first_of("=")+1);
					}
					if(buf.find_first_of(";")<string::npos)
					{
						wert = wert.substr(0,wert.size()-1);
					}
					// Zuweisungen:
					if(variable=="ntax")
					{
						cout << " dimensions " << "ntax=" << atoi(wert.c_str()) <<endl; 
						CurrentData->setNTaxa(atoi(wert.c_str()));
					}
					else if(variable=="nchar")
					{
						cout << " dimensions " << "nchar=" << atoi(wert.c_str()) <<endl;
						CurrentData->setNChar(atoi(wert.c_str()));
					}
					if(buf.find_first_of(";")<string::npos)
						break;
				}
			}// ende dimensions
			else if(buf=="format")
			{
				while(true)
				{
					StringStream >> buf;
					if(buf==";")break;
					string variable = buf.substr(0,buf.find_first_of("=;"));
					string wert = "";
					if(buf.find_first_of("=")<string::npos) //falls = zeichen enthalten...
					{
						wert = buf.substr(buf.find_first_of("=")+1);
					}
					if(buf.find_first_of(";")<string::npos)
					{
						wert = wert.substr(0,wert.size()-1);
					}
					// Zuweisungen:
					if(variable=="missing")
					{
						cout << " format " << "missing=" << wert.at(0) <<endl;
						CurrentData->setMissing(wert.at(0));
					}
					else if(variable=="gap")
					{
						cout << " format " << "gap=" << wert.at(0) <<endl;
						CurrentData->setGap(wert.at(0));
					}
					else if(variable=="matchchar")
					{
						cout << " format " << "matchchar=" << wert.at(0) <<endl;
						CurrentData->setMatching(wert.at(0));
					}
					else if(variable=="datatype")
					{
						//std::cout<< " datatype: "	<< "\t= "	<< wert <<std::endl;
						//CurrentData->set...
					}
					else if(variable=="interleave")
					{
						interleave = true;
					}
					if(buf.find_first_of(";")<string::npos)
						break;
				}
			}// ende format
			else if(buf=="options")
			{
			}// ende options
			else if(buf=="matrix")
			{
				if(interleave)
				{
					cout << " species in data-matrix:" << endl;
					while(std::getline(StringStream,buf))
					{
						if( buf.find_first_of(";") < string::npos )
							break;
						std::stringstream TempStringStream;
						TempStringStream << buf;
						TempStringStream >> buf;
						string name = buf;
						string characters = "";
						while(TempStringStream >> buf)
						{
							characters += buf;
						}
						//std::cout << name << "\t"<<characters << std::endl; //Name Characters
						if(name != "")
						{
							CurrentData->insertSpeciesData(name,characters);
							//cout << " - " << name << endl;
						}
						//cout << name << endl;
					}
				}
				else
				{
					cout << " species in data-matrix:" << endl;
					for(int i=0;i<CurrentData->getNTaxa();++i)// todo anzahl spezien einsetzen.
					{
						// Name der Spezies einlesen:
						StringStream >> buf;
						string name = buf;
						// alle Characters zu der Spezies einlesen
						int nchar = CurrentData->getNChar();// todo anzahl char einsetzen.
						string characters = "";
						while(nchar>0)
						{
							StringStream >> buf;
							characters += buf;
							nchar -= buf.size();
						}
						//std::cout << name << " "<<characters << std::endl; //Name Characters
						CurrentData->insertSpeciesData(name,characters);
						//cout << " - " << name << endl;
					}
				}
				// active : all species are included in the analysis
				vector<int> iactive;
				for(int i=(CurrentData->getNTaxa()-1); i>=0 ;--i)
				{
					iactive.push_back(i);
				}	
				CurrentData->setActiveSpecies(iactive);
				map<string,int> tempSpecies	= CurrentData->getSpecies2Id();
				map<string,int>::iterator iter;
				for( iter = tempSpecies.begin(); iter != tempSpecies.end(); ++iter )
				{
					cout << " -  " << iter->first << endl;
				}
			}//ende matrix
			StringStream >> buf;
		}
		cout << "Parsing data-block finished." << endl;
	}



	void NexusParser::processNexusTreeTimeBlock(std::stringstream& StringStream)
	{
		cout << "Parsing treetime-block:" << endl;
		string buf;
		StringStream >> buf;
		while(buf!="end;" && buf!="End;" && buf!="END;")
		{
			if(buf=="mcmcmc")
			{
				while(true)
				{
					StringStream >> buf;
					if(buf==";")break;
					string variable = buf.substr(0,buf.find_first_of("=;"));
					string wert = "";
					if(buf.find_first_of("=")<string::npos) //falls = zeichen enthalten...
					{
						wert = buf.substr(buf.find_first_of("=")+1);
					}
					if(buf.find_first_of(";")<string::npos)
					{
						wert = wert.substr(0,wert.size()-1);
					}
					// Zuweisungen:
					if(variable=="nChains")
					{
						cout << " mcmcmc " << "nChains=" << atoi(wert.c_str()) <<endl; 
						CurrentData->setNChains(atoi(wert.c_str()));
					}
					else if(variable=="nSamples")
					{
						cout << " mcmcmc " << "nSamples=" << atoi(wert.c_str()) <<endl; 
						CurrentData->setNSamples(atoi(wert.c_str()));
					}
					else if(variable=="burnInPeriod")
					{
						cout << " mcmcmc " << "burnInPeriod=" << atoi(wert.c_str()) <<endl; 
						CurrentData->setBurnIn(atoi(wert.c_str()));
					}
					else if(variable=="sampleFrequency")
					{
						cout << " mcmcmc " << "sampleFrequency=" << atoi(wert.c_str()) <<endl; 
						CurrentData->setSampleFrequency(atoi(wert.c_str()));
					}
					else if(variable=="heatingCoefficient")
					{
						cout << " mcmcmc " << "heatingCoefficient=" << atof(wert.c_str()) <<endl; 
						CurrentData->setHeatingCoefficent(atof(wert.c_str()));
					}
					if(buf.find_first_of(";")<string::npos)
						break;
				}
			}// ende mcmcmc
			else if(buf=="setting")
			{
				while(true)
				{
					StringStream >> buf;
					if(buf==";")break;
					string variable = buf.substr(0,buf.find_first_of("=;"));
					string wert = "";
					if(buf.find_first_of("=")<string::npos) //falls = zeichen enthalten...
					{
						wert = buf.substr(buf.find_first_of("=")+1);
					}
					if(buf.find_first_of(";")<string::npos)
					{
						wert = wert.substr(0,wert.size()-1);
					}
					// Zuweisungen:
					if(variable=="seed")
					{
						cout << " setting " << "seed=" << atoi(wert.c_str()) <<endl; 
						CurrentData->setSeed(atoi(wert.c_str()));
					}
					else if(variable=="rateChanges")
					{
						cout << " setting " << "rateChanges=" << wert <<endl;
						if(wert == "0")
							CurrentData->setRateChanges(false);
						else
							CurrentData->setRateChanges(true);
					}
					else if(variable=="memChanges")
					{
						cout << " setting " << "memChanges=" << wert <<endl;
						if(wert == "0")
							CurrentData->setMemChanges(false);
						else
							CurrentData->setMemChanges(true);
					}
					else if(variable=="outputPath")
					{
						cout << " setting " << "outputPath=" << wert <<endl;
						CurrentData->setOutputPath(wert);
					}
					if(buf.find_first_of(";")<string::npos)
					{
						break;
					}
				}
			}// end setting
			else if(buf=="parameter")
			{
				while(true)
				{
					StringStream >> buf;
					if(buf==";")break;
					string variable = buf.substr(0,buf.find_first_of("=;"));
					string wert = "";
					if(buf.find_first_of("=")<string::npos) //falls = zeichen enthalten...
					{
						wert = buf.substr(buf.find_first_of("=")+1);
					}
					if(buf.find_first_of(";")<string::npos)
					{
						wert = wert.substr(0,wert.size()-1);
					}
					// Zuweisungen:
					if(variable=="cNodeTime")
					{
						cout << " parameter " << "cNodeTime=" << wert <<endl;
						CurrentData->setNodeTime(atof(wert.c_str()));
					}
					else if(variable=="cTimeSynchronisation")
					{
						cout << " parameter " << "cTimeSynchronisation=" << wert <<endl;
						CurrentData->setTimeSynchronisation(atof(wert.c_str()));
					}
					else if(variable=="cRootTime")
					{
						cout << " parameter " << "cRootTime=" << wert <<endl;
						CurrentData->setRootTime(atof(wert.c_str()));
					}
					if(buf.find_first_of(";")<string::npos)
					{
						break;
					}
				}
			}// end parameter
			else if(buf=="weighting")
			{
				while(true)
				{
					StringStream >> buf;
					if(buf==";")break;
					string variable = buf.substr(0,buf.find_first_of("=;"));
					string wert = "";
					if(buf.find_first_of("=")<string::npos) //falls = zeichen enthalten...
					{
						wert = buf.substr(buf.find_first_of("=")+1);
					}
					if(buf.find_first_of(";")<string::npos)
					{
						wert = wert.substr(0,wert.size()-1);
					}
					// Zuweisungen:
					if(variable=="topology")
					{
						cout << " weighting " << "topology=" << wert <<endl;
						CurrentData->setWeightTopology(atof(wert.c_str()));
					}
					else if(variable=="nodes")
					{
						cout << " weighting " << "nodes=" << wert <<endl;
						CurrentData->setWeightNodes(atof(wert.c_str()));
					}
					else if(variable=="evolution")
					{
						cout << " weighting " << "evolution=" << wert <<endl;
						CurrentData->setWeightEvolution(atof(wert.c_str()));
					}
					else if(variable=="ratechange")
					{
						cout << " weighting " << "ratechange=" << wert <<endl;
						CurrentData->setWeightRatechange(atof(wert.c_str()));
					}
					else if(variable=="timeconversion")
					{
						cout << " weighting " << "timeconversion=" << wert <<endl;
						CurrentData->setWeightTimeconversion(atof(wert.c_str()));
					}
					if(buf.find_first_of(";")<string::npos)
					{
						break;
					}
				}
			}// end weighting
			else if(buf=="initialTree")
			{
				while(true)
				{
					StringStream >> buf;
					if(buf==";")break;
					string variable = buf.substr(0,buf.find_first_of("=;"));
					string wert = "";
					if(buf.find_first_of("=")<string::npos) //falls = zeichen enthalten...
					{
						wert = buf.substr(buf.find_first_of("=")+1);
					}
					if(buf.find_first_of(";")<string::npos)
					{
						wert = wert.substr(0,wert.size()-1);
					}
					// Zuweisungen:
					if(variable=="tree")
					{
						cout << " initialTree " << "tree=" << wert <<endl;
						CurrentData->setInitialTree(wert+";");
					}
					if(buf.find_first_of(";")<string::npos)
						break;
				}
			}// ende initialTree
			else if(buf=="taxa")
			{
				while(true)
				{
					StringStream >> buf;
					if(buf==";")break;
					string variable = buf.substr(0,buf.find_first_of("=;"));
					string wert = "";
					if(buf.find_first_of("=")<string::npos) //falls = zeichen enthalten...
					{
						wert = buf.substr(buf.find_first_of("=")+1);
					}
					if(buf.find_first_of(";")<string::npos)
					{
						wert = wert.substr(0,wert.size()-1);
					}
					// Zuweisungen:
					if(variable=="outgroup")
					{
						cout << " taxa " << "outgroup=" << wert <<endl;
						int id = (CurrentData->getSpecies2Id())[wert];
						CurrentData->setOutgroup(id);
//						// todo constraint setzen:
//						set<int> outgroupId;
//						set<int> restIds;
//						outgroupId.insert(id);
//						for(map<string,int>::iterator iS = 
//								CurrentData->getSpecies2Id().begin();
//								iS != CurrentData->getSpecies2Id().end(); ++iS)
//						{
//							if(iS->second != id)
//							{
//								restIds.insert(iS->second);
//							}
//						}
//						CurrentData->insertConstraint(outgroupId,restIds,-1,-1);
					}
					else if(variable=="active")
					{
						cout << " taxa " << "active: all species are included in the analysis" <<endl;
						/*
						vector<string> active;
						wert = wert.substr(1,wert.size()-2);
						if(wert!="")wert.append(",");
						while(wert!="")
						{
							active.push_back(wert.substr(0,wert.find_first_of(",")));
							wert = wert.substr(wert.find_first_of(",")+1);
						}
						vector<int> iactive;
						for(unsigned int i = 0; i<active.size(); ++i)
						{
							if( ((CurrentData->getSpecies2Id()).find(active[i])) != ((CurrentData->getSpecies2Id()).end()))
								iactive.push_back(  (CurrentData->getSpecies2Id())[active[i]] );
						}
						CurrentData->setActiveSpecies(iactive);
						*/
					}
					if(buf.find_first_of(";")<string::npos)
						break;
				}
			}// ende taxa
			else if(buf=="topologicalConstraint")
			{
				cout << " topologicalConstraint";
				vector<string> list1;
				vector<string> list2;
				while(true)
				{
					StringStream >> buf;
					if(buf==";")
					{
						break;
					}
					string variable = buf.substr(0,buf.find_first_of("=;"));
					string wert = "";
					if(buf.find_first_of("=")<string::npos) //falls = zeichen enthalten...
					{
						wert = buf.substr(buf.find_first_of("=")+1);
					}
					if(buf.find_first_of(";")<string::npos)
					{
						wert = wert.substr(0,wert.size()-1);
					}
					// Zuweisungen:
					if(variable=="list1")
					{
						cout << " list1=" << wert;
						wert = wert.substr(1,wert.size()-2);
						if(wert!="")wert.append(",");
						while(wert!="")
						{
							list1.push_back(wert.substr(0,wert.find_first_of(",")));
							wert = wert.substr(wert.find_first_of(",")+1);
						}
					}
					else if(variable=="list2")
					{
						cout << " list2=" << wert;
						wert = wert.substr(1,wert.size()-2);
						if(wert!="")wert.append(",");
						while(wert!="")
						{
							list2.push_back(wert.substr(0,wert.find_first_of(",")));
							wert = wert.substr(wert.find_first_of(",")+1);
						}
					}
					if(buf.find_first_of(";")<string::npos)
					{
						break;
					}
				}
				cout << endl;
				set<int> ilist1;
				set<int> ilist2;
				for(unsigned int i = 0; i<list1.size(); ++i)
				{
					if( ((CurrentData->getSpecies2Id()).find(list1[i])) != ((CurrentData->getSpecies2Id()).end()))
						ilist1.insert(  (CurrentData->getSpecies2Id())[list1[i]] );
				}
				for(unsigned int i = 0; i<list2.size(); ++i)
				{
					if( ((CurrentData->getSpecies2Id()).find(list2[i])) != ((CurrentData->getSpecies2Id()).end()))
						ilist2.insert(  (CurrentData->getSpecies2Id())[list2[i]] );
				}
				CurrentData->insertTopologicalConstraint(ilist1,ilist2);
			}// ende topologicalConstraint
			else if(buf=="gaussianConstraint")
			{
				cout << " gaussianConstraint";
				vector<string> list1;
				vector<string> list2;
				double mean = -1;
				double sd   = -1;
				while(true)
				{
					StringStream >> buf;
					if(buf==";")
					{
						break;
					}
					string variable = buf.substr(0,buf.find_first_of("=;"));
					string wert = "";
					if(buf.find_first_of("=")<string::npos) //falls = zeichen enthalten...
					{
						wert = buf.substr(buf.find_first_of("=")+1);
					}
					if(buf.find_first_of(";")<string::npos)
					{
						wert = wert.substr(0,wert.size()-1);
					}
					// Zuweisungen:
					if(variable=="list1")
					{
						cout << " list1=" << wert;
						wert = wert.substr(1,wert.size()-2);
						if(wert!="")wert.append(",");
						while(wert!="")
						{
							list1.push_back(wert.substr(0,wert.find_first_of(",")));
							wert = wert.substr(wert.find_first_of(",")+1);
						}
					}
					else if(variable=="list2")
					{
						cout << " list2=" << wert;
						wert = wert.substr(1,wert.size()-2);
						if(wert!="")wert.append(",");
						while(wert!="")
						{
							list2.push_back(wert.substr(0,wert.find_first_of(",")));
							wert = wert.substr(wert.find_first_of(",")+1);
						}
					}
					else if(variable=="mean")
					{
						cout << " mean=" << wert;
						mean = (atof(wert.c_str()));
					}
					else if(variable=="sd")
					{
						cout << " sd=" << wert;
						sd = (atof(wert.c_str()));
					}
					if(buf.find_first_of(";")<string::npos)
					{
						break;
					}
				}
				cout << endl;
				set<int> ilist1;
				set<int> ilist2;
				for(unsigned int i = 0; i<list1.size(); ++i)
				{
					if( ((CurrentData->getSpecies2Id()).find(list1[i])) != ((CurrentData->getSpecies2Id()).end()))
						ilist1.insert(  (CurrentData->getSpecies2Id())[list1[i]] );
				}
				for(unsigned int i = 0; i<list2.size(); ++i)
				{
					if( ((CurrentData->getSpecies2Id()).find(list2[i])) != ((CurrentData->getSpecies2Id()).end()))
						ilist2.insert(  (CurrentData->getSpecies2Id())[list2[i]] );
				}
				CurrentData->insertGaussianConstraint(ilist1,ilist2,mean,sd);
			}// ende gaussianConstraint
			else if(buf=="gammaConstraint")
			{
				cout << " gammaConstraint";
				vector<string> list1;
				vector<string> list2;
				double min		= -1;
				double mean		= -1;
				double alpha	=  2;
				double threshold= 1.0E-300;
				while(true)
				{
					StringStream >> buf;
					if(buf==";")
					{
						break;
					}
					string variable = buf.substr(0,buf.find_first_of("=;"));
					string wert = "";
					if(buf.find_first_of("=")<string::npos) //falls = zeichen enthalten...
					{
						wert = buf.substr(buf.find_first_of("=")+1);
					}
					if(buf.find_first_of(";")<string::npos)
					{
						wert = wert.substr(0,wert.size()-1);
					}
					// Zuweisungen:
					if(variable=="list1")
					{
						cout << " list1=" << wert;
						wert = wert.substr(1,wert.size()-2);
						if(wert!="")wert.append(",");
						while(wert!="")
						{
							list1.push_back(wert.substr(0,wert.find_first_of(",")));
							wert = wert.substr(wert.find_first_of(",")+1);
						}
					}
					else if(variable=="list2")
					{
						cout << " list2=" << wert;
						wert = wert.substr(1,wert.size()-2);
						if(wert!="")wert.append(",");
						while(wert!="")
						{
							list2.push_back(wert.substr(0,wert.find_first_of(",")));
							wert = wert.substr(wert.find_first_of(",")+1);
						}
					}
					else if(variable=="min")
					{
						cout << " min=" << wert;
						min = (atof(wert.c_str()));
					}
					else if(variable=="mean")
					{
						cout << " mean=" << wert;
						mean = (atof(wert.c_str()));
					}
					else if(variable=="alpha")
					{
						cout << " alpha=" << wert;
						alpha = (atof(wert.c_str()));
					}
					else if(variable=="threshold")
					{
						cout << " threshold=" << wert;
						threshold = (atof(wert.c_str()));
					}
					if(buf.find_first_of(";")<string::npos)
					{
						break;
					}
				}
				cout << endl;
				set<int> ilist1;
				set<int> ilist2;
				for(unsigned int i = 0; i<list1.size(); ++i)
				{
					if( ((CurrentData->getSpecies2Id()).find(list1[i])) != ((CurrentData->getSpecies2Id()).end()))
						ilist1.insert(  (CurrentData->getSpecies2Id())[list1[i]] );
				}
				for(unsigned int i = 0; i<list2.size(); ++i)
				{
					if( ((CurrentData->getSpecies2Id()).find(list2[i])) != ((CurrentData->getSpecies2Id()).end()))
						ilist2.insert(  (CurrentData->getSpecies2Id())[list2[i]] );
				}
				CurrentData->insertGammaConstraint(ilist1,ilist2,min,mean,alpha,threshold);
			}// ende gammaConstraints
			else if(buf=="dnaData")
			{
				int nSites;
				int nLoci;
				//vector<int> beginningFragments;
				vector<int>& beginningFragments = CurrentData->getBeginningFragments();
				while(true)
				{
					StringStream >> buf;
					if(buf==";")break;
					string variable = buf.substr(0,buf.find_first_of("=;"));
					string wert = "";
					if(buf.find_first_of("=")<string::npos) //falls = zeichen enthalten...
					{
						wert = buf.substr(buf.find_first_of("=")+1);
					}
					if(buf.find_first_of(";")<string::npos)
					{
						wert = wert.substr(0,wert.size()-1);
					}
					// Zuweisungen:
					if(variable=="nSites")
					{
						cout << " dnaData " << "nSites=" << wert <<endl;
						nSites = atoi(wert.c_str());
					}
					else if(variable=="nLoci")
					{
						cout << " dnaData " << "nLoci=" << wert <<endl;
						nLoci=atoi(wert.c_str());
					}
					else if(variable=="beginningFragments")
					{
						cout << " dnaData " << "beginningFragments=" << wert <<endl;
						wert.append(",");
						while(wert!="")
						{
							beginningFragments.push_back(atoi((wert.substr(0,wert.find_first_of(","))).c_str()));
							wert = wert.substr(wert.find_first_of(",")+1);
						}
					}
					else if(variable=="models")
					{
						cout << " dnaData " << "models=" << wert <<endl;
						wert.append(",");
						while(wert!="")
						{
							int i = wert.find_first_of(")");
							CurrentData->insertDnaEvolutionModel(wert.substr(0,i+1));
							wert = wert.substr(i+2);
						}
					}
					if(buf.find_first_of(";")<string::npos)
						break;
				}
			}// ende dnaData
			else if(buf=="rateChange")
			{
				while(true)
				{
					StringStream >> buf;
					if(buf==";")break;
					string variable = buf.substr(0,buf.find_first_of("=;"));
					string wert = "";
					if(buf.find_first_of("=")<string::npos) //falls = zeichen enthalten...
					{
						wert = buf.substr(buf.find_first_of("=")+1);
					}
					if(buf.find_first_of(";")<string::npos)
					{
						wert = wert.substr(0,wert.size()-1);
					}
					// Zuweisungen:
					if(variable=="globalModel")
					{
						cout << " rateChange " << "globalModel=" << wert <<endl;
						CurrentData->getInitialGlobalRateChangeModel() = wert;
					}
					else if(variable=="localModel")
					{
						cout << " rateChange " << "localModel=" << wert <<endl;
						CurrentData->getInitialRateChangeModels().clear();
						wert.append(",");
						while(wert!="")
						{
							int i = wert.find_first_of(")");
							CurrentData->getInitialRateChangeModels().push_back(wert.substr(0,i+1));
							wert = wert.substr(i+2);
						}
					}
					if(buf.find_first_of(";")<string::npos)
						break;
				}
			}// ende rateChange
			StringStream >> buf;
		}
		cout << "Parsing treetime-block finished." << endl;
	}
	
	
	
	
	
	
	
	
	
	
	
	
	/*void NexusParser::parseNexusFile2(string& nexusfile)
	{
		std::ifstream ifs(nexusfile.c_str());
		string buf;
		while (ifs >> buf)
		{
			std::cout << buf <<std::endl;
		}
	}*/
	
	
	int NexusParser::positionOfOpeningBracket(string& str)
	{
		if(str.find_first_of("[")>=string::npos)// nicht drin
		{
			return string::npos; 
		}
		else if(str.find_first_of("[")==0)
		{
			return 0;
		}
		else
		{
			string x = str;
			int pos = 0;
			while(x.find_first_of("[") < string::npos)//wenn vorhanden
			{
				int pos_ = x.find_first_of("[");
				pos += pos_;
				if(str.at(pos-1) == '\"' || str.at(pos-1) == '\'')
				{
					x = x.substr(pos_+1);
					pos++;
				}
				else
				{
					return pos;
				}
			}
			return string::npos;
		}
	}
	
	int NexusParser::positionOfClosingBracket(string& str)
	{
		if(str.find_first_of("]")>=string::npos)// nicht drin
		{
			return string::npos; 
		}
		else if(str.find_first_of("]")==0)
		{
			return 0;
		}
		else
		{
			string x = str;
			int pos = 0;
			while(x.find_first_of("]") < string::npos)//wenn vorhanden
			{
				int pos_ = x.find_first_of("]");
				pos += pos_;
				if(str.at(pos-1) == '\"' || str.at(pos-1) == '\'')
				{
					x = x.substr(pos_+1);
					pos++;
				}
				else
				{
					return pos;
				}
			}
			return string::npos;
		}
	}
	
	int NexusParser::positionOfBracket(string& str)
	{
		if(str.find_first_of("[]")>=string::npos)// nicht drin
		{
			return -1; 
		}
		else if(str.find_first_of("[]")==0)
		{
			return 0;
		}
		else
		{
			string x = str;
			int pos = 0;
			while(x.find_first_of("[]") < string::npos)//wenn vorhanden
			{
				int pos_ = x.find_first_of("[]");
				pos += pos_;
				if(str.at(pos-1) == '\"' || str.at(pos-1) == '\'')
				{
					x = x.substr(pos_+1);
					pos++;
				}
				else
				{
					return pos;
				}
			}
			return -1;
		}
	}
	
	void NexusParser::strToUpperCase(string& str)
	{
		for(unsigned int i=0; i<str.size(); ++i)
		{
			str.replace(i,1,1,toupper(str.at(i)));
		}
	}


}//TreeTime_NexusProcessing
