/*
Copyright (C) 2006-2008 Lin Himmelmann

This file is part of TreeTime.
See the NOTICE file distributed with this work for additional
information regarding copyright ownership and licensing.

TreeTime is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

TreeTime is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with TreeTime.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef SYMDISTMATRIX_HPP_
#define SYMDISTMATRIX_HPP_

#include "Settings.hpp"

namespace TreeTime_NexusProcessing
{

	class SymDistMatrix
	{
		private:
			const unsigned int	m_dim;
			double**			m_data;

		public:
			SymDistMatrix(unsigned int p_dim, double p_init=0);
			SymDistMatrix(const SymDistMatrix&);
			SymDistMatrix& operator=(const SymDistMatrix&);
			~SymDistMatrix();
			
			SymDistMatrix& operator +=(const SymDistMatrix&);
			SymDistMatrix& operator -=(const SymDistMatrix&);
			SymDistMatrix& operator *=(double);
			SymDistMatrix& operator /=(double);
			const SymDistMatrix operator +(const SymDistMatrix&);
			const SymDistMatrix operator -(const SymDistMatrix&);
			
			double& 		operator()(unsigned int,unsigned int);
			const double&	operator()(unsigned int,unsigned int) const;
			double			calcEuclideanDistance(const SymDistMatrix&) const;
			unsigned int 	getDimension() const;
			void			print();
			void			print() const;
			
			pair<int,int>	getMinimum() const;
			pair<int,int>	getMaximum() const;
			
			double			getMin() const;
			double			getMax() const;
	};//SymDistMatrix

}//TreeTime_NexusProcessing

#endif /*SYMDISTMATRIX_HPP_*/
