/*
Copyright (C) 2006-2008 Lin Himmelmann

This file is part of TreeTime.
See the NOTICE file distributed with this work for additional
information regarding copyright ownership and licensing.

TreeTime is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

TreeTime is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with TreeTime.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef UPGMA_HPP_
#define UPGMA_HPP_

#include "Settings.hpp"

namespace TreeTime_NexusProcessing
{
	
	class SymDistMatrix;
	
	class Upgma
	{
		public:
			class UpgmaCluster;
		
		private:
			const SymDistMatrix* m_pDistanceMatrix;
		
		public:
			Upgma(const SymDistMatrix* const);
			~Upgma();
			
		private:
			Upgma(const Upgma&);
			Upgma& operator=(const Upgma&);
		
		public:
			void			setDistanceMatrix(const SymDistMatrix* const);
			string			calcNewickString(vector<string>* = 0) const;
			
		private:
			void			calcNewickString(const UpgmaCluster*, string&,double, vector<string>*) const;
			UpgmaCluster*	startUpgma() const;
			UpgmaCluster*	mergeCluster(const UpgmaCluster*, const UpgmaCluster*, double) const;
			pair<int,int>	getMin(const SymDistMatrix&, const map<int,UpgmaCluster*>&) const;
			void			updateDistances(SymDistMatrix& ,map<int,UpgmaCluster*>&, const UpgmaCluster* const, int) const;
		
		public:
			class UpgmaCluster
			{
				private:
					double						m_height;
					set<int>					m_sIds;
					list<const UpgmaCluster*>	m_lChildCluster;
						
				public:
					UpgmaCluster(double p_height);
					UpgmaCluster(double p_height, int p_id);
					~UpgmaCluster();
				
				private:
					UpgmaCluster(const UpgmaCluster&);
					UpgmaCluster& operator=(const UpgmaCluster&);
					
				public:
					double								getHeight() const;
					void								insertChildCluster(const UpgmaCluster*);
					const set<int>&						getIds() const;
					void								insertIds(const set<int>&);
					const list<const UpgmaCluster*>&	getChildCluster() const;
					double								calcDistance(const UpgmaCluster*, const SymDistMatrix&) const;
			};
	};

}//TreeTime_NexusProcessing

#endif /*UPGMA_HPP_*/
