/*
Copyright (C) 2006-2008 Lin Himmelmann

This file is part of TreeTime.
See the NOTICE file distributed with this work for additional
information regarding copyright ownership and licensing.

TreeTime is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

TreeTime is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with TreeTime.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef CPPCHANGERATEINTENSITY_HPP_
#define CPPCHANGERATEINTENSITY_HPP_

#include "../../TreeTime_Mcmcmc/Change.hpp"
#include "../../TreeTime_Core/State.hpp"

namespace TreeTime_RateChangeModel
{
	template<class STATE>
	class CompoundPoissonProcess;

	class CppChangeRateIntensity : public TreeTime_Mcmcmc::Change
	{
		typedef TreeTime_MolecularTree::Tree<TreeTime_Core::NodeData>
				Phylogeny;
		typedef TreeTime_MolecularTree::iterator<TreeTime_Core::NodeData>
				Iterator;
		typedef TreeTime_Core::State State;

		public:

			CppChangeRateIntensity(State* p_pState, double p_proposalRatio,
					int p_loci, double p_expPriorParameter,
					double p_calibration);
			virtual ~CppChangeRateIntensity();
			virtual void	changeState();
			virtual double	getStationaryDistributionRatio() const;
			virtual double	getProposalRatio() const;
			virtual void	accept();
			virtual void	reject();
			virtual int		getId () const;

		private:

			State*				m_pState;
			const int			m_loci;
			double				m_proposalRatio;
			// The Exponential Prior Parameter
			// for the Intenity of Rate Change Poisson Process
			// (1/m_expPriorParameter) is the mean value of the Intensity.
			// The smaller m_expPriorParameter is,
			// the more Change Points are favoured. 
			double				m_expPriorParameter;
			CompoundPoissonProcess<State>*	m_pCurrentCpp;
			CompoundPoissonProcess<State>*	m_pProposalCpp;
			double				m_proposalRateLogLikelihood;
			double				m_currentRateLogLikelihood;
			double				m_ratioMultiplier;
			double				m_calibration;

	};//CppChangeRateIntensity

}//TreeTime_RateChangeModel

#endif /*CPPCHANGERATEINTENSITY_HPP_*/
