/*
Copyright (C) 2006-2008 Lin Himmelmann

This file is part of TreeTime.
See the NOTICE file distributed with this work for additional
information regarding copyright ownership and licensing.

TreeTime is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

TreeTime is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with TreeTime.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef CPPCHANGERATELOCATION_HPP_
#define CPPCHANGERATELOCATION_HPP_

#include "../../TreeTime_Mcmcmc/Change.hpp"
#include "../../TreeTime_Core/State.hpp"

namespace TreeTime_RateChangeModel
{

	class CppChangeRateLocation : public TreeTime_Mcmcmc::Change
	{

		typedef TreeTime_MolecularTree::Tree<TreeTime_Core::NodeData>
				Phylogeny;
		typedef TreeTime_MolecularTree::iterator<TreeTime_Core::NodeData>
				Iterator;
		typedef TreeTime_Core::State State;

		public:

			CppChangeRateLocation(State* p_pState, double p_proposalRatio,
					int p_loci);
			virtual ~CppChangeRateLocation();
			virtual void	changeState();
			virtual double	getStationaryDistributionRatio() const;
			virtual double	getProposalRatio() const;
			virtual void	accept();
			virtual void	reject();
			virtual int		getId () const;

		private:

			void allocateSubtreeMemory(Iterator p_iterator);
			void calcRateMultiplier(Iterator p_iterator, int p_loci);

		private:

			State*					m_pState;
			const int				m_loci;
			double					m_proposalRatio;
			Phylogeny*				m_pProposalTree;
			double					m_proposalTreeLogLikelihood;
			list<realf****>			m_lpCurrentProbablities;
			list<realf****>			m_lpProposalProbablities;

	};//CppChangeRateLocation

}//TreeTime_RateChangeModel

#endif /*CPPCHANGERATELOCATION_HPP_*/
