/*
Copyright (C) 2006-2008 Lin Himmelmann

This file is part of TreeTime.
See the NOTICE file distributed with this work for additional
information regarding copyright ownership and licensing.

TreeTime is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

TreeTime is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with TreeTime.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef DIRICHLETMODEL_HPP_
#define DIRICHLETMODEL_HPP_

#include "Settings.hpp"
#include "RateChangeModel.hpp"
#include "../TreeTime_MolecularTree/iterator.hpp"

namespace TreeTime_RateChangeModel
{

	template<class STATE>
	class DirichletModel;

	template<class STATE>
	std::ostream& operator<<(std::ostream&, const DirichletModel<STATE>&);

	template<class STATE>
	class DirichletModel : public RateChangeModel<STATE>
	{

		friend ostream& operator<< <STATE>(ostream&,
				const DirichletModel<STATE>&);

		public:

			DirichletModel(double p_expectedVariance, double p_initialVariance,
					int p_nSpecies, double p_changeCalibrationVariance, double
					p_changeCalibrationAll, double p_changeCalibrationTwo);
			virtual ~DirichletModel();
			DirichletModel(const DirichletModel& p_Rhs);
			DirichletModel& operator= (const DirichletModel& p_Rhs);

			virtual DirichletModel<STATE>* clone() const;
			virtual TreeTime_Mcmcmc::Change* getChange(STATE* p_pState,
					int p_loci, double p_runiform) const;
			virtual double getLogDensity(STATE* p_pState, int p_loci) const;
			virtual void initRateChanges(STATE* p_pState, int p_loci) const;
			virtual int getId() const;

			inline int		getNRateMultipliers() const;
			inline double	getVariance() const;
			inline void		setVariance(double p_variance);

		private:

			/**
			 * Output Operator, which is called by the namespace level output
			 * operator, and is deduced to the concrete classes.
			 */
			 virtual void operator<<(ostream&) const;

		private:

			// Variance of the Dirichlet Distribution [Variance for a Rate]
			// in [0.01,0.1] => std deviation in [0.1,0.316]
			double		m_variance;
			// Number of Dimensions in the Dirichlet Distribution
			const int	m_nRates;
			// Exponential Prior to the Variance
			// in [0.01,0.05]
			const double		m_expectedVariance;
			// Parameters to control the Changes
			const double 		m_changeCalibrationVariance;
			const double 		m_changeCalibrationAll;
			const double 		m_changeCalibrationTwo;

	};//DirichletModel

}//TreeTime_RateChangeModel

#include "DirichletModel.cxx"

#endif /*DIRICHLETMODEL_HPP_*/
