/*
Copyright (C) 2006-2008 Lin Himmelmann

This file is part of TreeTime.
See the NOTICE file distributed with this work for additional
information regarding copyright ownership and licensing.

TreeTime is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

TreeTime is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with TreeTime.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef MOLECULARCLOCK_HPP_
#define MOLECULARCLOCK_HPP_

#include "Settings.hpp"
#include "RateChangeModel.hpp"

namespace TreeTime_RateChangeModel
{
	template<class STATE>
	class MolecularClock;//Forward Declaration

	/**
	 * PrintStream for Output of RateChangeModel,
	 * implemented as a namespace level function.
	 * @param ostream&
	 * @param const DnaEvolutionModel&
	 * @return ostream&
	 */
	template<class STATE>
	ostream& operator<<(ostream& , const MolecularClock<STATE>&);

	/// Represents the abstract Interface for a RateChangeModel
	/**
	 * The Evolution Model consists of a Stationary Distribution
	 */
	template<class STATE>
	class MolecularClock : public RateChangeModel<STATE>
	{
		/**
		 * Declaration of friend ouput operator
		 */
		friend ostream& operator<< <STATE>(ostream&, const MolecularClock<STATE>&);

		public:
	
			/**
			 * Constructor
			 */
			MolecularClock();
			/**
			 * Destructor of DnaEvolutionModel
			 */
			virtual ~MolecularClock();

			/**
			 * Copy Constructor
			 */
			MolecularClock(const MolecularClock<STATE>& p_Rhs);
			/**
			 * Assignment Operator
			 */
			MolecularClock<STATE>& operator=(const MolecularClock<STATE>& rhs);

			/**
			 * Returns a Clone of the Dna Evolution Model
			 * @return DnaEvolutionModel* 
			 */
			virtual MolecularClock<STATE>* clone() const;

			/**
			 */
			virtual TreeTime_Mcmcmc::Change* getChange(STATE* p_pTree,
					int p_loci, double p_runiform) const;
			/**
			 * Sets the Random Generator for changing the Model Parametrers
			 */
			virtual double getLogDensity(STATE* p_pTree, int p_loci= -1) const;
			
			virtual void initRateChanges(STATE* p_pState, int p_loci) const;

			virtual int getId() const;

		private:

			/**
			 * Output Operator, which is called by the namespace level output
			 * operator, and is deduced to the concrete classes.
			 */
			 virtual void operator<<(ostream&) const;

	};//MolecularClock

}//TreeTime_RateChangeModel

#include "MolecularClock.cxx"

#endif /*MOLECULARCLOCK_HPP_*/
