/*
Copyright (C) 2006-2008 Lin Himmelmann

This file is part of TreeTime.
See the NOTICE file distributed with this work for additional
information regarding copyright ownership and licensing.

TreeTime is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

TreeTime is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with TreeTime.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "MolecularClock.hpp"
#include "CompoundPoissonProcess.hpp"
#include "DirichletModel.hpp"
#include "UncorrelatedExponential.hpp"
#include "UncorrelatedLognormal.hpp"

namespace TreeTime_RateChangeModel
{
	//forward declarations:
	template<class STATE>
	class CompoundPoissonProcess;

	template<class STATE>
	class MolecularClock;


	template<class STATE>
	Random* RateChangeModel<STATE>::s_pRandom;

	template<class STATE>
	RateChangeModel<STATE>::RateChangeModel()
	{
	}

	template<class STATE>
	RateChangeModel<STATE>::RateChangeModel(const RateChangeModel<STATE>& p_Rhs)
	{
	}

	template<class STATE>
	RateChangeModel<STATE>::~RateChangeModel()
	{
	}

	template<class STATE>
	RateChangeModel<STATE>* RateChangeModel<STATE>::newRateChangeModel(
			string p_stringRepresentation)
	{
		if(p_stringRepresentation == "")
		{
			return new MolecularClock<STATE>();
		}
		string model = p_stringRepresentation.substr(0,
				p_stringRepresentation.find_first_of("("));
		vector<double> par(7,1.0);
		p_stringRepresentation = p_stringRepresentation.substr(
				p_stringRepresentation.find_first_of("("));
		p_stringRepresentation = p_stringRepresentation.
				substr(1, p_stringRepresentation.size()-2);
		if(p_stringRepresentation.size()>0)
			p_stringRepresentation.append(",");
		int index = 0;
		while(p_stringRepresentation!="")
		{
			int nextComma = p_stringRepresentation.find_first_of(",");
			par[index] = atof((p_stringRepresentation.substr(0,nextComma)).
					c_str());
			p_stringRepresentation = p_stringRepresentation.substr(nextComma+1);
			++index;
		}
		if(model=="CPP")
		{
			return new CompoundPoissonProcess<STATE>(par[0], par[1], par[2],
					par[3], par[4], par[5], par[6]);
		}
		else if(model=="DM")
		{
			return new DirichletModel<STATE>(par[0], par[1], static_cast<int>
					(par[2]), par[3], par[4], par[5]);
		}
		else if(model=="ULN")
		{
			return new UncorrelatedLognormal<STATE>(par[0], par[1],
					static_cast<int>(par[2]), par[3], par[4], par[5]);
		}
		else if(model=="UEX")
		{
			return new UncorrelatedExponential<STATE>(static_cast<int>(par[0]),
					par[1], par[2]);
		}
		else
		{
			return new MolecularClock<STATE>();
		}
	}

	template<class STATE>
	void RateChangeModel<STATE>::operator<<(std::ostream& p_out) const
	{
		p_out << "RCM";
	}

	template<class STATE>
	std::ostream& operator<<(std::ostream& p_out,
			const RateChangeModel<STATE>& p_Rcm)
	{
		p_Rcm.operator<<(p_out);
		return p_out;
	}

	template<class STATE>
	inline void RateChangeModel<STATE>::setRandom(Random* p_pRandom)
	{
		s_pRandom = p_pRandom;
	}

}//TreeTime_RateChangeModel

