/*
Copyright (C) 2006-2008 Lin Himmelmann

This file is part of TreeTime.
See the NOTICE file distributed with this work for additional
information regarding copyright ownership and licensing.

TreeTime is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

TreeTime is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with TreeTime.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef UNCORRELATEDEXPONENTIAL_HPP_
#define UNCORRELATEDEXPONENTIAL_HPP_

#include "Settings.hpp"
#include "RateChangeModel.hpp"

namespace TreeTime_RateChangeModel
{

	template<class STATE>
	class UncorrelatedExponential;

	template<class STATE>
	std::ostream& operator<<(std::ostream&, const
			UncorrelatedExponential<STATE>&);

	template<class STATE>
	class UncorrelatedExponential : public RateChangeModel<STATE>
	{

		friend ostream& operator<< <STATE>(ostream&,
				const UncorrelatedExponential<STATE>&);

		public:

			UncorrelatedExponential(int p_nSpecies, double
					p_changeCalibrationAll, double p_changeCalibrationSingle);
			virtual ~UncorrelatedExponential();
			UncorrelatedExponential(const UncorrelatedExponential&);
			UncorrelatedExponential& operator= (const UncorrelatedExponential&);

			virtual UncorrelatedExponential<STATE>* clone() const;
			virtual TreeTime_Mcmcmc::Change* getChange(STATE* p_pState,
					int p_loci, double p_runiform) const;
			virtual double getLogDensity(STATE* p_pState, int p_loci) const;
			virtual void initRateChanges(STATE* p_pState, int p_loci) const;
			virtual int getId() const;

		private:

			/**
			 * Output Operator, which is called by the namespace level output
			 * operator, and is deduced to the concrete classes.
			 */
			 virtual void operator<<(ostream&) const;

		private:

			const int	m_nRates;
			double 		m_changeCalibrationAll;
			double 		m_changeCalibrationSingle;

	};//UncorrelatedExponential

}//TreeTime_RateChangeModel

#include "UncorrelatedExponential.cxx"


#endif /*UNCORRELATEDEXPONENTIAL_HPP_*/
