/*
Copyright (C) 2006-2008 Lin Himmelmann

This file is part of TreeTime.
See the NOTICE file distributed with this work for additional
information regarding copyright ownership and licensing.

TreeTime is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

TreeTime is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with TreeTime.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef UNCORRELATEDLOGNORMAL_HPP_
#define UNCORRELATEDLOGNORMAL_HPP_

#include "Settings.hpp"
#include "RateChangeModel.hpp"

namespace TreeTime_RateChangeModel
{

	template<class STATE>
	class UncorrelatedLognormal;

	template<class STATE>
	std::ostream& operator<<(std::ostream&,const UncorrelatedLognormal<STATE>&);

	template<class STATE>
	class UncorrelatedLognormal : public RateChangeModel<STATE>
	{

		friend ostream& operator<< <STATE>(ostream&,
				const UncorrelatedLognormal<STATE>&);

		public:

			UncorrelatedLognormal(double p_expectedVariance, double
					p_initialVariance, int p_nSpecies, double
					p_changeCalibrationVariance, double p_changeCalibrationAll,
					double p_changeCalibrationSingle);
			virtual ~UncorrelatedLognormal();
			UncorrelatedLognormal(const UncorrelatedLognormal&);
			UncorrelatedLognormal& operator= (const UncorrelatedLognormal&);

			virtual UncorrelatedLognormal<STATE>* clone() const;
			virtual TreeTime_Mcmcmc::Change* getChange(STATE* p_pState,
					int p_loci, double p_runiform) const;
			virtual double getLogDensity(STATE* p_pState, int p_loci) const;
			virtual void initRateChanges(STATE* p_pState, int p_loci) const;
			virtual int getId() const;

			inline double	getNormalMean() const;
			inline double	getNormalDeviation() const;
			inline double	getVariance() const;
			inline void		setVariance(double p_variance);

		private:

			/**
			 * Output Operator, which is called by the namespace level output
			 * operator, and is deduced to the concrete classes.
			 */
			 virtual void operator<<(ostream&) const;

		private:

			const int	m_nRates;
			double 		m_changeCalibrationVariance;
			double 		m_changeCalibrationAll;
			double 		m_changeCalibrationSingle;

			double		m_priorExpectedVariance; // [0.002, 0.02]
			double		m_variance; //= exp(m_normalVariance)-1
									// => m_normalVariance = log (m_variance+1)
									// => m_normalMean = - m_normalVariance/2
			double		m_normalVariance;
			double		m_normalMean;
			double		m_normalDeviation;

	};//UncorrelatedLognormal

}//TreeTime_RateChangeModel

#include "UncorrelatedLognormal.cxx"

#endif /*UNCORRELATEDLOGNORMAL_HPP_*/
